-- Filename:     xor4nands.vhd
-- Filetype:     VHDL code (structural)
-- Date:         16 mar 2011
-- Update:       3 feb 2013
-- Description:  A structural description of an EXOR
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2013, De Haagse Hogeschool
--
-- Dit is een voorbeeld van een structural beschrijving
-- van een EXOR opgebouwd uit vier 2-input NANDS. Zie voor
-- meer informatie http://en.wikipedia.org/wiki/XOR_gate
--

library ieee;
use ieee.std_logic_1164.all;

-- Beschrijving van de signalen van de EXOR
entity xor4nands is
    port (
        X: in std_logic;
        Y: in std_logic;
        S: out std_logic
    );
end xor4nands;

-- Architecture van de EXOR
architecture structural of xor4nands is

-- Declaratie van de te gebruiken NAND. In feite is
-- dit de entity-beschrijving maar nu is het keyword
-- entity vervangen door component.
component Nand_2 is
    port (
        P: in std_logic;
        Q: in std_logic;
        R: out std_logic
    );
end component;

-- Interne signalen om de diverse in- en uitgangen
-- van de NANDs aan elkaar te koppelen (glue signals)
signal A,B,C : std_logic;

-- In de structural beschrijving worden vier NANDs
-- geinstantieerd. De in- en uitgangen van de NANDs
-- worden gekoppeld aan interne signalen (glue signals)
-- of aan de in- en uitgangen van de EXOR. Dit gebeurt
-- door middel van een port map constructie. De werking:
--   instantienaam:  componentnaam
--       port map (componentsignal => hogerehierachiesignal, ...)
-- 
begin
	U1: Nand_2 
       port map (P => X, Q => Y, R => A);
	U2: Nand_2
       port map (P => X, Q => A, R => B);
	U3: Nand_2
       port map (P => A, Q => Y, R => C);
	U4: Nand_2
       port map (P => B, Q => C, R => S);
end structural;
